@@ -26,6 +26,7 @@ from .baseapp import (
     PIDFileError,
     base_flags, base_aliases
 )
+from .._version import __version__
 from .daemonize import daemonize
 
 
@@ -39,7 +40,7 @@ _description = """Start an IPython cluster for parallel computing.
 An IPython cluster consists of 1 controller and 1 or more engines.
 This command automates the startup of these processes using a wide range of
 startup methods (SSH, local processes, PBS, mpiexec, SGE, LSF, HTCondor,
-Windows HPC Server 2008). To start a cluster with 4 engines on your
+Slurm, Windows HPC Server 2008). To start a cluster with 4 engines on your
 local host simply do 'ipcluster start --n=4'. For more complex usage
 you will typically do 'ipython profile create mycluster --parallel', then edit
 configuration files, followed by 'ipcluster start --profile=mycluster --n=4'.
@@ -95,7 +96,7 @@ def find_launcher_class(clsname, kind):
     clsname : str
         The full name of the launcher class, either with or without the
         module path, or an abbreviation (MPI, SSH, SGE, PBS, LSF, HTCondor
-        WindowsHPC).
+        Slurm, WindowsHPC).
     kind : str
         Either 'EngineSet' or 'Controller'.
     """
@@ -264,6 +265,7 @@ class IPClusterEngines(BaseParallelApplication):
                         around, so you will likely have to do this manually
                         unless the machines are on a shared file system.
             HTCondor : use HTCondor to submit engines to a batch queue
+            Slurm : use Slurm to submit engines to a batch queue
             WindowsHPC : use Windows HPC
 
         If you are using one of IPython's builtin launchers, you can specify just the
@@ -288,7 +290,7 @@ class IPClusterEngines(BaseParallelApplication):
 
     early_shutdown = Integer(30, config=True, help="The timeout (in seconds)")
     _stopping = False
-    
+
     aliases = Dict(engine_aliases)
     flags = Dict(engine_flags)
 
@@ -320,9 +322,10 @@ class IPClusterEngines(BaseParallelApplication):
         return launcher
 
     def engines_started_ok(self):
-        self.log.info("Engines appear to have started successfully")
-        self.early_shutdown = 0
-    
+        if self.engine_launcher.running:
+            self.log.info("Engines appear to have started successfully")
+            self.early_shutdown = 0
+
     def start_engines(self):
         # Some EngineSetLaunchers ignore `n` and use their own engine count, such as SSH:
         n = getattr(self.engine_launcher, 'engine_count', self.n)
@@ -340,21 +343,21 @@ class IPClusterEngines(BaseParallelApplication):
         if self.early_shutdown and not self._stopping:
             self.log.error("""
             Engines shutdown early, they probably failed to connect.
-            
+
             Check the engine log files for output.
-            
+
             If your controller and engines are not on the same machine, you probably
             have to instruct the controller to listen on an interface other than localhost.
-            
+
             You can set this by adding "--ip='*'" to your ControllerLauncher.controller_args.
-            
+
             Be sure to read our security docs before instructing your controller to listen on
             a public interface.
             """)
             self.stop_launchers()
-        
+
         return self.engines_stopped(r)
-    
+
     def engines_stopped(self, r):
         return self.loop.stop()
 
@@ -399,7 +402,7 @@ class IPClusterEngines(BaseParallelApplication):
         if self.daemonize:
             if os.name=='posix':
                 daemonize()
-        
+
         self.loop.add_callback(self.start_engines)
         # Now write the new pid file AFTER our new forked pid is active.
         # self.write_pid_file()
@@ -440,7 +443,7 @@ class IPClusterStart(IPClusterEngines):
 
     delay = CFloat(1., config=True,
         help="delay (in s) between starting the controller and the engines")
-    
+
     controller_ip = Unicode(config=True, help="Set the IP address of the controller.")
     controller_launcher = Any(config=True, help="Deprecated, use controller_launcher_class")
     def _controller_launcher_changed(self, name, old, new):
@@ -456,7 +459,7 @@ class IPClusterStart(IPClusterEngines):
 
         Each launcher class has its own set of configuration options, for making sure
         it will work in your environment.
-        
+
         Note that using a batch launcher for the controller *does not* put it
         in the same batch job as the engines, so they will still start separately.
 
@@ -468,6 +471,7 @@ class IPClusterStart(IPClusterEngines):
             SGE : use SGE (qsub) to submit the controller to a batch queue
             LSF : use LSF (bsub) to submit the controller to a batch queue
             HTCondor : use HTCondor to submit the controller to a batch queue
+            Slurm : use Slurm to submit engines to a batch queue
             SSH : use SSH to start the controller
             WindowsHPC : use Windows HPC
 
@@ -494,11 +498,11 @@ class IPClusterStart(IPClusterEngines):
         if self.controller_ip:
             self.controller_launcher.controller_args.append('--ip=%s' % self.controller_ip)
         self.engine_launcher = self.build_launcher(self.engine_launcher_class, 'EngineSet')
-    
+
     def engines_stopped(self, r):
         """prevent parent.engines_stopped from stopping everything on engine shutdown"""
         pass
-    
+
     def start_controller(self):
         self.log.info("Starting Controller with %s", self.controller_launcher_class)
         self.controller_launcher.on_stop(self.stop_launchers)
@@ -546,7 +550,7 @@ class IPClusterStart(IPClusterEngines):
         if self.daemonize:
             if os.name=='posix':
                 daemonize()
-        
+
         def start():
             self.start_controller()
             self.loop.add_timeout(self.loop.time() + self.delay, self.start_engines)
@@ -568,27 +572,39 @@ class IPClusterStart(IPClusterEngines):
 
 class IPClusterNBExtension(BaseIPythonApplication):
     """Enable/disable ipcluster tab extension in Jupyter notebook"""
-    
+
     name = 'ipcluster-nbextension'
-    
-    description = """Enable/disable IPython clusters tab in Jupyter notebook"""
-    
+
+    description = """Enable/disable IPython clusters tab in Jupyter notebook
+
+    for Jupyter Notebook >= 4.2, you can use the new nbextension API:
+
+    jupyter serverextension enable --py ipyparallel
+    jupyter nbextension install --py ipyparallel
+    jupyter nbextension enable --py ipyparallel
+    """
+
     examples = """
     ipcluster nbextension enable
     ipcluster nbextension disable
     """
-    
+    version = __version__
+    user = Bool(False, help="Apply the operation only for the given user").tag(
+        config=True)
+    flags = Dict({'user': ({'IPClusterNBExtension': {'user': True}},
+                           'Apply the operation only for the given user')})
+
     def start(self):
-        from ipyparallel.nbextension.install import install_server_extension
+        from ipyparallel.nbextension.install import install_extensions
         if len(self.extra_args) != 1:
             self.exit("Must specify 'enable' or 'disable'")
         action = self.extra_args[0].lower()
         if action == 'enable':
             print("Enabling IPython clusters tab")
-            install_server_extension(enable=True)
+            install_extensions(enable=True, user=self.user)
         elif action == 'disable':
             print("Disabling IPython clusters tab")
-            install_server_extension(enable=False)
+            install_extensions(enable=False, user=self.user)
         else:
             self.exit("Must specify 'enable' or 'disable', not '%s'" % action)
 
@@ -599,6 +615,7 @@ class IPClusterApp(BaseIPythonApplication):
     name = u'ipcluster'
     description = _description
     examples = _main_examples
+    version = __version__
 
     subcommands = {
                 'start' : (base+'Start', start_help),
@@ -613,7 +630,9 @@ class IPClusterApp(BaseIPythonApplication):
 
     def start(self):
         if self.subapp is None:
-            print("No subcommand specified. Must specify one of: %s"%(self.subcommands.keys()))
+            keys = ', '.join("'{}'".format(key) for
+                             key in self.subcommands.keys())
+            print("No subcommand specified. Must specify one of: %s" % keys)
             print()
             self.print_description()
             self.print_subcommands()
@@ -625,4 +644,3 @@ launch_new_instance = IPClusterApp.launch_instance
 
 if __name__ == '__main__':
     launch_new_instance()
-
